import { expect, tap } from '@pushrocks/tapbundle';
import * as typedrequest from '@apiglobal/typedrequest';
import * as typedrequestInterfaces from '@apiglobal/typedrequest-interfaces';

import * as typedsocket from '../ts/index';
import { request } from 'http';

interface IRequest_Client_Server extends typedrequestInterfaces.implementsTR<
  typedrequestInterfaces.ITypedRequest,
  IRequest_Client_Server
> {
  method: 'sayhi';
  request: {
    greeting: string;
  };
  response: {
    answer: string
  }
}

let testTypedSocketServer: typedsocket.TypedSocket;
let testTypedSocketClient: typedsocket.TypedSocket;

const testTypedRouter = new typedrequest.TypedRouter();

tap.test('should add some handlers', async () => {
  testTypedRouter.addTypedHandler<IRequest_Client_Server>(new typedrequest.TypedHandler('sayhi', async requestData => {
    return {
      answer: `ok, got it : ${requestData.greeting}`
    }
  }));
})

tap.test('should create Server and Client', async (tools) => {
  testTypedSocketServer = await typedsocket.TypedSocket.createServer(testTypedRouter);
  testTypedSocketClient = await typedsocket.TypedSocket.createClient(testTypedRouter, 'http://localhost:3000');
});

tap.test('should process messages from both sides', async () => {
  const myServerSideTypedRequest = testTypedSocketServer.createTypedRequest<IRequest_Client_Server>('sayhi');
  const myClientSideTypedRequest = testTypedSocketClient.createTypedRequest<IRequest_Client_Server>('sayhi');
  const response = await myClientSideTypedRequest.fire({
    greeting: 'that is a greeting from the client'
  });
  console.log(response);
  const response2 = await myServerSideTypedRequest.fire({
    greeting: 'that is a greeting from the server'
  });
  console.log(response2);
})

tap.test('should run without problems for a little bit', async tools => {
  await tools.delayFor(5000);
})

tap.test('should disconnect', async () => {
  await testTypedSocketClient.stop();
  await testTypedSocketServer.stop();
})

tap.start();
